==> pam_require-0.2.tgz: pam_require-0.2/pam_require.c <==
/*
pam_require - A simple PAM account module
Copyright (C) 2003 Andreas Gohr <a.gohr@web.de>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <security/pam_modules.h>
#include <syslog.h>
#include <string.h>
#include <sys/types.h>
#include <grp.h>
#include <pwd.h>

int check_groups (pam_handle_t*,const char*,const char*);

/* --------------------------- PAM functions --------------------------------*/

/* function for account modules - The only one this module handles */
PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh,
                                int flags,
                                int argc,
                                const char **argv){
  const char *user;
  int rval;
  int i;
  user = NULL;

  /* get username from PAM or fail */
  if ( (rval = pam_get_user(pamh,&user,NULL)) != PAM_SUCCESS ){
    syslog( LOG_ERR, "pam_require: can't get username: %s",
                     pam_strerror(pamh, rval) );
    return PAM_AUTH_ERR;
  }

  /* check all arguments */
  for (i=0; i<argc; i++){
    /* skip standard params */
    if (strcmp(argv[i],"debug") == 0){ continue; }
    if (strcmp(argv[i],"no_warn") == 0){ continue; }
    if (strcmp(argv[i],"use_first_pass") == 0){ continue; }
    if (strcmp(argv[i],"try_first_pass") == 0){ continue; }
    if (strcmp(argv[i],"use_mapped_pass") == 0){ continue; }
    if (strcmp(argv[i],"expose_account") == 0){ continue; }

    if (argv[i][0] == '@'){
      /* check if user is member of required group */
      if (check_groups (pamh,user,argv[i]+1)){
        return PAM_SUCCESS;
      }
    }else{
      /* check if username matches */
      if (strcmp(argv[i],user) == 0){
        return PAM_SUCCESS;
      }
    }
  }
  
  /* authfail when arriving here */
  return PAM_AUTH_ERR;
}

/* function for auth modules - ignore us! */
PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh,
                                   int flags,
                                   int argc,
                                   const char **argv){
  return PAM_IGNORE;
}

/* function for password modules - ignore us! */
PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh,
                                int flags,
                                int argc,
                                const char **argv){
  return PAM_IGNORE;
}

/* functions for session modules - ignor us */
PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh,
                                   int flags,
                                   int argc,
                                   const char **argv){
  return PAM_IGNORE;
}
PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh,
                                    int flags,
                                    int argc,
                                    const char **argv){
  return PAM_IGNORE;
}

/* --------------------------- my own functions -----------------------------*/

int check_groups (pam_handle_t *pamh,const char *user,const char *groupname){
  int i;
  int ngroups = 16;
  struct group *grp;
  struct passwd *pw;
  gid_t *groups;

  /* find GID for required group */
  grp = getgrnam (groupname);
  if (grp == NULL){
    syslog( LOG_ERR, "pam_require: Group %s does not exist", groupname );
    return 0;
  }

  /* get user infos */
  pw = getpwnam (user);
  if (pw == NULL){
    syslog( LOG_ERR, "pam_require: Could not get userinfos for %s", user );
    return 0;
  }

  /* get list of groups the user is in */
  groups = (gid_t *) malloc (ngroups * sizeof (gid_t));
  if (getgrouplist (pw->pw_name, pw->pw_gid, groups, &ngroups) < 0) {
    groups = (gid_t *) realloc (ngroups * sizeof (gid_t));
    getgrouplist (pw->pw_name, pw->pw_gid, groups, &ngroups);
  }

  /* check if required group is in the list */
  for (i=0;i<ngroups;i++){
    if (groups[i] == grp->gr_gid){
      free(groups);
      return 1;
    }
  }
  free(groups);
  return 0;
}

/* setup Vim: ex: set et ts=2 tw=80 : */

==> pam_require-0.3.tgz: pam_require-0.3/pam_require.c <==
/*
pam_require - A simple PAM account module
Copyright (C) 2003 Andreas Gohr <a.gohr@web.de>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <security/pam_modules.h>
#include <syslog.h>
#include <string.h>
#include <sys/types.h>
#include <grp.h>
#include <pwd.h>

int check_groups (pam_handle_t*,const char*,const char*);

/* --------------------------- PAM functions --------------------------------*/

/* function for account modules - The only one this module handles */
PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh,
                                int flags,
                                int argc,
                                const char **argv){
  const char *user;
  int rval;
  int i;
  user = NULL;

  /* get username from PAM or fail */
  if ( (rval = pam_get_user(pamh,&user,NULL)) != PAM_SUCCESS ){
    syslog( LOG_ERR, "pam_require: can't get username: %s",
                     pam_strerror(pamh, rval) );
    return PAM_AUTH_ERR;
  }

  /* check all arguments */
  for (i=0; i<argc; i++){
    /* skip standard params */
    if (strcmp(argv[i],"debug") == 0){ continue; }
    if (strcmp(argv[i],"no_warn") == 0){ continue; }
    if (strcmp(argv[i],"use_first_pass") == 0){ continue; }
    if (strcmp(argv[i],"try_first_pass") == 0){ continue; }
    if (strcmp(argv[i],"use_mapped_pass") == 0){ continue; }
    if (strcmp(argv[i],"expose_account") == 0){ continue; }

    if (argv[i][0] == '!'){
      if (argv[i][1] == '@'){
        /* check if user is _not_ member of required group */
        if (!check_groups (pamh,user,argv[i]+2)){
          return PAM_SUCCESS;
        }
      }else{
        /* check if username does _not_ match */
        if (strcmp(argv[i]+1,user) != 0){
          return PAM_SUCCESS;
        }
      }
    } else if (argv[i][0] == '@'){
      /* check if user is member of required group */
      if (check_groups (pamh,user,argv[i]+1)){
        return PAM_SUCCESS;
      }
    }else{
      /* check if username matches */
      if (strcmp(argv[i],user) == 0){
        return PAM_SUCCESS;
      }
    }
  }
  
  /* authfail when arriving here */
  return PAM_AUTH_ERR;
}

/* function for auth modules - ignore us! */
PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh,
                                   int flags,
                                   int argc,
                                   const char **argv){
  return PAM_IGNORE;
}

/* function for password modules - ignore us! */
PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh,
                                int flags,
                                int argc,
                                const char **argv){
  return PAM_IGNORE;
}

/* functions for session modules - ignor us */
PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh,
                                   int flags,
                                   int argc,
                                   const char **argv){
  return PAM_IGNORE;
}
PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh,
                                    int flags,
                                    int argc,
                                    const char **argv){
  return PAM_IGNORE;
}

/* --------------------------- my own functions -----------------------------*/

int check_groups (pam_handle_t *pamh,const char *user,const char *groupname){
  int i;
  int ngroups = 16;
  struct group *grp;
  struct passwd *pw;
  gid_t *groups;

  /* find GID for required group */
  grp = getgrnam (groupname);
  if (grp == NULL){
    syslog( LOG_ERR, "pam_require: Group %s does not exist", groupname );
    return 0;
  }

  /* get user infos */
  pw = getpwnam (user);
  if (pw == NULL){
    syslog( LOG_ERR, "pam_require: Could not get userinfos for %s", user );
    return 0;
  }

  /* get list of groups the user is in */
  groups = (gid_t *) malloc (ngroups * sizeof (gid_t));
  if (getgrouplist (pw->pw_name, pw->pw_gid, groups, &ngroups) < 0) {
    groups = (gid_t *) realloc (ngroups * sizeof (gid_t));
    getgrouplist (pw->pw_name, pw->pw_gid, groups, &ngroups);
  }

  /* check if required group is in the list */
  for (i=0;i<ngroups;i++){
    if (groups[i] == grp->gr_gid){
      free(groups);
      return 1;
    }
  }
  free(groups);
  return 0;
}

/* setup Vim: ex: set et ts=2 tw=80 : */
